;This is a small demo program for the Creativision.
;
;It's coded in 6502 assembler and should be compiled with DASM. It doesn't need any .h files,
;but the files PatternGeneratorTable.asm and ColorTable.asm should be in the same directory as this one.
;
;The correct command line for calling DASM is:
;dasm.exe createst.asm -lcreatest_lst.txt -f3 -v5 -ocreatest.bin
;This creates createst_lst.txt as a list file and createst.bin as the binary.
;
;The Creativision has a 6502 processor (same as the Atari 2600),
;a TMS9928 video chip mapped to addresses $2000 (data read), $2001 (status read), $3000 (data write) and $3001 (status write)
;and a SN76489 sound chip mapped to address $1002 (write).
;The remainder is unimportant at the moment.
;
;This program demonstrates how to work with the VDP (Video chip) and the PSG (soundchip) by doing the following:
;- Displaying a full bitmap on the VDP in bitmap mode (that's how the mode is called in the official TI documentation, e.g. the Editor/Assembler manual for the TI-99).
;- Playing some music
;It doesn't demonstrate how to read the keyboard or joysticks and how to react to that input.
;
;- The bitmap gets displayed by copying the color table and pattern table into the VDP RAM and filling the pattern name table with 3 times 00-FF.
;- The sound generation is interrupt driven. It plays a melody by reading out a melody table. This table contains 3 bytes for each 8th note of the song.
;  Each value read defines a note playing through one voice or a pause. So the three bytes per 8th note together describe the notes for the 3 voices
;  in each 8th note.
;  Since the frequency data resolution of the sound generators actually is 10 bits, I created a frequency table containing high and low bytes for each
;  of 36 notes which correspond to the halftone scale of 3 octaves.
;  Each byte in the melody table now either is an index into this table, or 254 denoting a pause, or 255 denoting that the melody should loop back to the start.
;  Each 8th note plays for 16 frames, and the playing notes are started at the loudest possible volume and then decay by 1 step per frame. Since the sound chip
;  has 16 possible volume values, it ends with silence before the next batch of notes get started.
;

	processor 6502
;	include vcs.h ;yes, this was lifted from Jim's heart will go on demo... but we don't need that here since we're doing a different system this time.

; Here come some equates specific to the Creativision:
;The following are the ports for reading and writing to the video and sound chips - you can find the descriptions of those chips on the Internet if you search
;for the part number (e.g. TMS9928 or SN76489) in a search engine - they basically do the same in every system they reside in, be it a TI-99, Colecovision, Creativision or others.

VDP_Status_Write = $3001
VDP_Data_Write = $3000
VDP_Data_Read = $2000
VDP_Status_Read = $2001
Sound_Write = $1002
Sound_Status = $1003

;The following are some interesting BIOS routines which we can use for our purposes.

BIOS_Sound_Write = $FE77 ;this is the address of a BIOS subroutine which writes a byte to the sound chip, then waits until it's not busy anymore
	;(which is probably necessary to do because on the Creativision, the sound chip is addressed through the PIA)
BIOS_Interrupt_Return = $FF52 ;this is another BIOS routine which returns from an interrupt - it does all the things like pulling back registers from the stack and finally the RTI.
BIOS_Interrupt_Handler = $FF3F ;this is the BIOS's internal interrupt handler; it saves all the registers, stores the VDP status byte in RAM and then jumps to the
				;cartridge-internal interrupt handler through vector $BFEA
BIOS_VDP_Regwrite = $FE1F ;this is a short subroutine which writes the contents of a into VDP register x. It can also be used for initializing the read/write address,
	;in this case a is the LSB of the address, the lower 6 bits of x are the MSB, bit 6 of x contains 0 or 1 (0 for read, 1 for write), and bit 7 of x has to be set to 1
	;(which gets converted to 0 by the BIOS routine)
BIOS_VDP_Regsetup = $F84B ;this is the BIOS routine which sets up the VDP registers to the values from location $BFF0 on. You can call this if you set some
	;registers to other values and want to reset them to the original ones

;The following are variables kept in RAM. Possible RAM locations are $00 through $3FF.

;For simplicity, we keep part of the ones we had on the Atari 2600 for the music there.

Temp       = $82		;this stores various temporary bytes, since the 3 registers aren't always enough.
SoundCount = $83	;this isn't used here... it was used by the Atari 2600 version as a pointer into sound data
DecayCount = $84	;this holds the current value for the volume of the sound (always the same for all playing notes and 0 for the non-playing ones).
			;Basically, this gets counted down to 0 by 1 each frame, and when it reaches zero, new notes get processed and it starts to count down again from 15.
SoundBytePointer = $85	;actually, $85 through $86 - this is the pointer into the melody data. It's a two-byte pointer now because the data is longer than one page.
NoteBeingPlayed = $87 	;$87 for SG 0, $88 for SG 1 and $89 for SG 2 - this holds the last note having been read for each sound generator and is basically there
			;in order to decide whether the volume of that generator should be set to the decay value or left at 0.
GraphicsPointer = $8A	;This is a two-byte pointer again, which gets used for reading the graphics data and writing it into the ROM. Note that we only need one pointer here
			;because the VDP automatically increases its write address with every byte written, so we only have to set its write address at the start of the loop.

;Here comes the start part.

	org $A000	;note that the code actually starts with graphics data at $8000, but since the BIN file is jumbled, I had to do it in a different way. 
			;See near the end of code for details...

;now comes the data for the VDP's Pattern Generator Table (basically 12K, the format is similar to that of the C-64... each byte describes a horizontal line of 8 pixels,
;but after 8 bytes it jumps back to the first line position and 1 byte to the right. Note that you don't have to do it this way... by arranging the pattern name table differently,
;you could also write up to 64 consecutive 8-pixel lines and then jump back... but you can't write all 192 existing lines.

	include PatternGeneratorTable.asm 

;The next location gets jumped to by the vector at $BFFC, that is, it's the start location of the cartridge.
;Note that the BIOS jumps here BEFORE displaying the logo... in order to display that, the start location would have to be set to $F808 where the logo gets displayed
;(or only the year, depending on the BIOS version and maybe the hardware?). That routine then would jump into the cartridge again, but this time through vector $BFE8.
;By doing it this way, we bypass the logo display.

Start

	SEI	;Disable interrupts until everything is set up

;The reason why we have to disable the interrupts is because the BIOS's own interrupt routine reads the VDP status byte before jumping into the cartridge.
;Unfortunately, this may come between two address writes. Theoretically, we could enable interrupts again as soon as the address got written and we're only
;doing VDP RAM writes anymore, but it can't hurt to leave them disabled. In this case, the music starts after the picture is complete, which takes about 7-8 frames in PAL.

;The following loop clears all of the Creativision's RAM.

	LDX #$FF
	LDA #00
ClearMemLoop:
	STA 0,X
	STA 0100,X
	STA 0200,X
	STA 0300,X
	DEX
	BNE ClearMemLoop

;Initialize Sound Pointer with start address of melody table
	LDA #$BD
	STA SoundBytePointer+1
	LDA #$00
	STA SoundBytePointer
;Initialize DecayCount
	LDA #$05
	STA DecayCount

;Next we write the picture into VDP RAM.
	LDA #$00		;First write Pattern Generator Table into VRAM at $0000
	STA VDP_Status_Write	;LSB of the write address
	LDA #$40
	STA VDP_Status_Write	;MSB and command of write address - set write address to $0000

	LDA #$00		;prepare pointer to graphics data
	STA GraphicsPointer
	LDA #$A0
	STA GraphicsPointer+1

	JSR GraphicsCopy	;do the actual copying

;Write Pattern Name Table - this should be three times 00 through FF. Since the pattern name table immediately follows the pattern generator table in VDP RAM,
;we don't have to set the write address again because the pointer in the VDP already points at it.
	
	LDX #$00
	LDY #$03	;3 x 256 bytes
VDP_Write_Outer_Loop	
	STX VDP_Data_Write
	INX
	BNE VDP_Write_Outer_Loop
	DEY
	BNE VDP_Write_Outer_Loop

;Write Color Table (this contains the colors in the same order as the Pattern name table, giving a foreground and a background color for each horizontal 8-pixel stripe).
	LDA #$00		
	STA VDP_Status_Write	;LSB of the write address
	LDA #$60
	STA VDP_Status_Write	;MSB and command of write address - set write address to $2000, the start of the color table.

	LDA #$00		;initialize the graphics pointer with $8000, the address where the color data actually starts (though for the assembler,
				;it starts at $C000... read about that near the end of the code)
	STA GraphicsPointer
	LDA #$80
	STA GraphicsPointer+1

	JSR GraphicsCopy	;do the copying
EndGraphicsLoop

;Now we have written all picture data to the VDP, we have to enable the display. We disabled it until now because
;otherwise the VDP will have problems to keep up with our stream of bytes, especially during writing the pattern name table.


	LDA #$E2		;new value including set screen active bit
	STA VDP_Status_Write
	LDA #$81		;write into register 1
	STA VDP_Status_Write	;see comments on that BIOS routine above

;And we're done with initializing... clear interrupts and get caught in an endless loop since all sound operations are interrupt driven ;-)

	CLI	;enable interrupts now
EndlessLoop1
	JMP EndlessLoop1

GraphicsCopy			;this subroutine copies data from RAM into the VDP RAM. It uses the vector GraphicsPointer for this.
				;Since we need this loop twice (for Pattern Generator Table and Color Table), we put it in this subroutine.
	LDX #$18		;prepare loop - we have to write $1800 bytes of data (that is, 6K for each loop)
	LDY #$00
GraphicsCopy1
	LDA (GraphicsPointer),Y
	STA VDP_Data_Write
	INY
	BNE GraphicsCopy1	;end of inner loop
	INC GraphicsPointer+1
	DEX
	BNE GraphicsCopy1
	RTS			;end outer loop

;Now comes the Internal_Interrupt_Handler
;Note: This gets jumped to by the 2nd interrupt entry in the BIOS (after saving registers on the stack).
;Here we have included the sound handler so that it gets executed once per frame.

Internal_Interrupt_Handler:

;first we count down the decay variable and copy it to the volumes of the sound generator which are currently playing a note.

	DEC DecayCount
	LDA DecayCount
;Now process volume for all three sound generators (the noise channel doesn't get used here)
	LDX #2
SoundGeneratorLoop
	LDA NoteBeingPlayed,X	;check if that sound generator plays a note
	CMP #$FF
	BEQ NoNotePlayed	;if not, skip the write for it
	LDA DecayCount	;prepare the write by packing the required data into the byte to be written
	EOR #$FF		;(see documentation of sound chip) - this turns 0000xxxx into 1111yyyy, where y = 15-x (and x is the desired volume).
	AND #$9F		;make this 1001yyyy... the required format to tell the sound generator a new volume value
	STA Temp
	TXA			;now we have 000000xx in a, where x is the sound generator number (0 through 3)
	ROL			;move PSG number in bits 5 and 6
	ROL
	ROL
	ROL
	ROL			;this makes it 0xx00000
	CLC
	ADC Temp		;now we add to that the byte we previously did, so we get 1xx1yyyy... the required format for the volume change.
	JSR BIOS_Sound_Write	;jump into the BIOS to actually write that byte
NoNotePlayed
	DEX
	CPX #$FF
	BNE SoundGeneratorLoop	;loop back to next generator if we're not done yet

HandleNewNotes	
	LDA DecayCount	;check if volume has reached 0, in which case we read new notes
	BNE EndInterrupt		;if not, end interrupt

;The following part handles the reading of new notes to be played

	LDA #$0F		;if we read new notes, we first reset the decay value to 15, which produces the highest possible volume.
	STA DecayCount
ReadNewNotes1
	LDX #2			;prepare loop for reading notes
ReadNewNotes
	STX Temp
	LDY #0
	LDA (SoundBytePointer),Y	;fetch note number to be played
	STA NoteBeingPlayed,X		;store it in variable for currently playing note
	CMP #$FE			;if it's FE, it isn't a note but the end of the table, and we loop back to the start
	BEQ ReturnToStart
	CMP #$FF			;if it's FF, it isn't a note but a pause, so no note gets played this time for that sound generator
	BEQ PlayNoNote
	ROL				;otherwise, we double the value since the frequency table has 2 bytes for each possible note.
	TAX				;x now contains index into frequency table for that note
	INX
	INX				;add 2 to that (why do we do this here? I can't remember...)
	LDA Temp			;load PSG number in order to create the correct command byte... here we have 000000xx, where x is the PSG number (0-2).
	ROL
	ROL
	ROL
	ROL
	ROL				;now in bits 5 and 6 (0xx00000)
	CLC
	ORA #$80			;set MSB to signal Frequency setting (-> 1xx00000)
	ADC FrequencyTable,X		;add low byte of desired note	(-> 1xx0ffff, which is the correct format for the first byte)
	JSR BIOS_Sound_Write		;write first byte to PSG
	LDA FrequencyTable+1,X		;load high byte of desired note (no processing needed)
	JSR BIOS_Sound_Write		;write it out to sound generator

PlayNoNote				;this is where we continue if this note doesn't get played
	INC SoundBytePointer		;increase the sound pointer lo-byte and, if necessary, the hi-byte too
	BNE Noincrease
	INC SoundBytePointer+1
Noincrease
	LDX Temp
	DEX
	CPX #$FF
	BNE ReadNewNotes		;read more notes if we haven't processed all sound generators yet
EndInterrupt
	JMP BIOS_Interrupt_Return	;end interrupt handling

;The following label gets jumped to, if note $FE gets read, which means "loop back to the start".
ReturnToStart				;init Sound Pointer again
	LDA #$BD
	STA SoundBytePointer+1
	LDA #$00
	STA SoundBytePointer
	JMP ReadNewNotes1		;here we restart the loop since the previously read bytes were invalid


	org $BC00	;here we'll store the frequency for each possible note. I've prepared the data so it doesn't need to be shifted anymore.
			;Actually, the order of the bytes is reversed since the "word" directive got used. Basically, the first byte given
			;(for instance the $3F out of $3F0F) contains the highest 6 bits in bits 0-5, and and 2nd byte contains the low 4 bits in bits 0-3.
			;That's the bit locations required by the sound chip.
			;The sound generator is clocked at 2 MHz, which means that the lowest achievable frequency should be 2000000/1023/32 = 61,09 Hz.
			;The note scale I used starts at this frequency and goes up by one half-tone each step. Actually, this isn't properly "tuned"... the
			;frequency of 61,09 HZ ist about 18 cents unter a "B". See more on that below in the note table.
			;Note: it's possible to get even lower frequencies by setting channel 3 to output "periodic noise" at the frequency of channel 2,
			;which actually makes it output a square wave with a 1/15 duty cycle at a frequency that's 1/15 of the "normal" channel 2 frequency,
			;which is a bit under 4 octaves lower. However, I don't use this trick here, but I did use it in a TI-99 game to give a "punchy" bass line.
FrequencyTable
	.word $3F0F  ;lowest note = C0; 00 111111 0000 1111 = 1023
	.word $3C06  ;C#0; 00 111100 0000 0110 = 966
	.word $380F ;D0;	00 111000 0000 1111 = 911
	.word $350C; D#0; 00 110101 0000 1100 = 860
	.word $320C; E0; 00 110010 0000 1100 = 812
	.word $2F0E; F0; 00 101111 0000 1110 = 766
	.word $2D03; F#0;00 101101 0000 0011 = 723
	.word $2A8B; G0;00 101010 0000 1011= 683
	.word $2804; G#0;00 101000 0000 0100 = 644
	.word $2600; A0;00 100110 0000 0000 = 608
	.word $230E; A#0;00 100011 0000 1110 = 574
	.word $210E; H0;00 100001 0000 1110 = 542
	.word $2000; C1;00 100000 0000 0000 = 512
	.word $1E03; C#1;00 011110 0000 0011 = 483
	.word $1C07; D1;00 011100 0000 0111 = 455
	.word $1A0E; D#1;00 011010 0000 1110 = 430
	.word $1906; E1;00 011001 0000 0110 = 406
	.word $170F; F1;00 010111 0000 1111 = 383
	.word $160A; G1;00 010110 0000 1010 = 362
	.word $1505; F#1;00 010101 0000 0101 = 341
	.word $1402; G#1;00 010100 0000 0010 = 322
	.word $1300; A1;00 010011 0000 0000 = 304
	.word $110F; A#1;00 010001 0000 1111 = 287
	.word $100F; H1;00 010000 0000 1111 = 271
	.word $1000; C2; = 256
	.word $0F01; C#2 = 241
	.word $0E04; D2  = 228
	.word $0D07; D#2  = 215
	.word $0C0B; E2  = 203
	.word $0C00; F2  = 192
	.word $0B05; F#2  = 181
	.word $0A0B; G2  = 171
	.word $0A01; G#2  = 161
	.word $0908; A2  = 152
	.word $0900; A#2  = 144
	.word $0807; H2 = 135

	org $BD00	;here we'll store the melody data; 255 means pause, 254 means back to start; all other are indexes into the frequency table
			;This means that a byte value of 0 plays about the lowest possible "C" note (actually, 18 cents below that, which is 64,73 Hz).
			;A value of 12 plays the "C" one octave above it and so on... the highest possible note is a A#2 (described as H2 in the table above)	
			;since I've encoded 36 note steps. I've often given the notes as "5+12+x" or something like that to somehow simulate the octaves and notes in that.
			;Yes, I know, I could also have defined constants like "C#1" for the note values, but I didn't feel like that.
			;I've added 5 at the start because the lowest note played is a G (relative to the main key of the melody, which in this case is actually F major,
			;so that that note is acually a "C". Yes, I know it's confusing, but this is how I perceive and memorize music).
			;Each line holds the an 1/8 note played on all three sound generators, each group of 8 lines thus is a measure. 
			;The spaces of two lines denote the boundaries of different "parts" of the melody.
	.byte 5+0, 5+12+4, 5+12+0
	.byte 5+7, 5+12+0, 5+12-5
	.byte 5+4, 5+12+0, 5+12-5
	.byte 5+7, 5+12+0, 5+12-5
	.byte 5+0, 5+12+4, 5+12+0
	.byte 5+7, 5+12+0, 5+12-5
	.byte 5+4, 5+12+0, 5+12-5
	.byte 5+7, 5+12+0, 5+12-5

	.byte 5+2, 5+12+5, 5+12+2
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+5, 5+12+4, 5+12
	.byte 5+7, 5+12+4, 5+12
	.byte 5+2, 5+12+2, 5+12-1
	.byte 5+7, 255, 255 
	.byte 5+5, 255, 255
	.byte 5+7, 255, 255

	.byte 5+2, 5+12+5, 5+12+2
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+5, 5+12+4, 5+12
	.byte 5+7, 5+12+4, 5+12
	.byte 5+2, 5+12+2, 5+12-1
	.byte 5+7, 255, 255
	.byte 5+5, 5+12+9, 5+12+5
	.byte 5+7, 255, 255

	.byte 5+2, 5+12+7, 5+12+4
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+5, 5+12+4, 5+12
	.byte 5+7, 5+12+2, 5+12-1
	.byte 5+0, 5+12+0, 5+12-5
	.byte 5+7, 255, 255
	.byte 5+4, 255, 255
	.byte 5+7, 255, 255


	.byte 5+0, 5+12+7, 255
	.byte 255, 255, 255
	.byte 255, 5+12+4, 255
	.byte 255, 5+12+0, 255
	.byte 5-3, 5+12+12, 255
	.byte 255, 255, 255
	.byte 255, 5+12+11, 255
	.byte 255, 5+12+9, 255

	.byte 5-5, 5+12+7, 255
	.byte 255, 255, 255
	.byte 255, 5+12+12, 255
	.byte 255, 5+12+4, 255
	.byte 5-1, 5+12+7, 255
	.byte 255, 5+12+2, 255
	.byte 5-3, 255, 255
	.byte 5-1, 255, 255


	.byte 5+0, 255, 255
	.byte 5+7, 5+12+7, 5+12+4
	.byte 5+4, 5+12+7, 5+12+4
	.byte 5+7, 5+12+7, 5+12+4
	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+7, 5+12+4, 5+12-0
	.byte 5+4, 5+12+2, 5+12-5
	.byte 5+7, 5+12+0, 255

	.byte 5+0, 255, 255
	.byte 5+5, 5+12+12, 5+12+9
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+12, 5+12+9
	.byte 5+0, 5+12+14, 5+12+11
	.byte 5+5, 5+12+12, 5+12+9
	.byte 5-3, 5+12+11, 5+12+7
	.byte 5+5, 5+12+9, 5+12+5

	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+7, 5+12+4, 5+12+0
	.byte 5+4, 255, 255
	.byte 5+7, 5+12+4, 5+12+0
	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+7, 255, 255
	.byte 5+4, 5+12+4, 5+12-0
	.byte 5+7, 255, 255

	.byte 5-1, 5+12+7, 5+12+2
	.byte 5+2, 5+12+2, 5+12-5
	.byte 5-5, 255, 255
	.byte 5+2, 255, 255
	.byte 5-3, 255, 255
	.byte 5+2, 255, 255
	.byte 5-1, 255, 255
	.byte 5+2, 255, 255

	.byte 5+0, 255, 255
	.byte 5+7, 5+12+4, 5+12
	.byte 5+4, 5+12+4, 5+12
	.byte 5+7, 5+12+5, 5+12+2
	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+7, 5+12+9, 5+12+5
	.byte 5+4, 5+12+11, 5+12+7
	.byte 5+7, 5+12+9, 5+12+5

	.byte 5+0, 255, 255
	.byte 5+5, 255, 255
	.byte 5-3, 255, 255
	.byte 5+5, 255, 255
	.byte 5+0, 255, 255
	.byte 5+5, 255, 255
	.byte 5-3, 255, 255
	.byte 5+5, 255, 255

	.byte 5+0, 5+12+5, 5+12
	.byte 5+5, 5+12+5, 5+12
	.byte 5-3, 5+12+7, 5+12
	.byte 5+5, 5+12+9, 5+12+5
	.byte 5+0, 255, 255
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+11, 5+12+7

	.byte 5-1, 255, 255
	.byte 5+2, 255, 255
	.byte 5-5, 255, 255
	.byte 5+2, 255, 255
	.byte 5-3, 255, 255
	.byte 5+2, 255, 255
	.byte 5-1, 5+12+7, 255
	.byte 5+2, 255, 255

	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+7, 255, 255
	.byte 5+4, 255, 255
	.byte 5+7, 255, 255
	.byte 5+0, 5+12+11, 5+12+4
	.byte 5+7, 255, 255
	.byte 5+4, 5+12+9, 5+12+4
	.byte 5+7, 5+12+7, 5+12+4

	.byte 5+0, 5+12+11, 5+12+7
	.byte 5+5, 255, 255
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 255, 255
	.byte 5+0, 255, 255
	.byte 5+5, 255, 255
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 255, 255

	.byte 5-1, 5+12+7, 5+12+5
	.byte 5+2, 255, 255
	.byte 5-5, 5+24+2, 5+12+7
	.byte 5+2, 255, 255
	.byte 5-3, 5+12+12, 5+12+5
	.byte 5+2, 255, 255
	.byte 5-1, 5+12+11, 5+12+5
	.byte 5+2, 255, 255

	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+4, 255, 255
	.byte 5-1, 255, 255
	.byte 5+4, 255, 255
	.byte 5-3, 255, 255
	.byte 5+4, 255, 255
	.byte 5-5, 255, 255
	.byte 5+4, 255, 255


	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24, 5+12+4
	.byte 5-5, 5+24+2, 5+12+5
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+2, 5+24+2, 5+12+5
	.byte 5-5, 5+12+7, 255

	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24, 5+12+4
	.byte 5-5, 5+24+2, 5+12+5
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 255, 255
	.byte 5+2, 5+24-1, 5+12+2
	.byte 5-5, 5+12+7, 255

	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 5+24, 5+12+4
	.byte 5-5, 5+24+2, 5+12+5
	.byte 5+4, 5+24+4, 5+12+7
	.byte 5+0, 5+24+5, 5+12+9
	.byte 5+5, 5+24+4, 5+12+7
	.byte 5-3, 5+24+2, 5+12+5
	.byte 5+5, 5+12+12, 5+12+4

	.byte 5-1, 5+24-1, 5+12+2
	.byte 5+2, 5+12+7, 5+12-1
	.byte 5-5, 5+24-3, 5+12
	.byte 5+2, 5+24-1, 5+12+2
	.byte 5+0, 5+24, 5+12+4
	.byte 5+4, 255, 255
	.byte 5-5, 5+24, 5+12+4
	.byte 5+4, 255, 255

	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 5+12+7, 5+12+4
	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+9, 5+12+5

	.byte 5+0, 5+12+7, 5+12+4
	.byte 5+4, 5+12+9, 5+12+5
	.byte 5-5, 5+12+7, 5+12+4
	.byte 5+4, 5+12+5, 5+12+2
	.byte 5+0, 5+12+4, 5+12
	.byte 5+4, 255, 255
	.byte 5-5, 5+12+7, 5+12+4
	.byte 5+4, 255, 255

	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+9, 5+12+5
	.byte 5+5, 5+12+7, 5+12+4
	.byte 5+0, 5+12+9, 5+12+5
	.byte 5+5, 5+12+11, 5+12+7
	.byte 5-3, 5+12+12, 5+12+9
	.byte 5+5, 5+12+9, 5+12+5

	.byte 5-1, 5+12+7, 5+12+2
	.byte 5+2, 5+12+12, 5+12+7
	.byte 5-5, 5+12+11, 5+12+5
	.byte 5+2, 5+12+14, 5+12+7
	.byte 5+0, 5+12+12, 5+12+4
	.byte 5+4, 255, 255
	.byte 5-5, 5+12+12, 5+12+4
	.byte 5+4, 255, 255

	.byte 254	 ;this loops back to the start



	org $BFEA	;this is the address where the BIOS' own interrupt handler expects a vector to the cartridge's own interrupt routine
			;The reason why we need this is because the BIOS has two interrupt handlers: a "short" and a "long" one.
			;The "short" one directly jumps to vector $BFFC, which may point somewhere into the cartridge or into the ROM.
			;In this demo (see below), I've pointed it at ROM. The routine pointed at saves the registers to the stack and then jumps here.
			;This means that after completing the interrupt handler, we have to jump to another BIOS routine which pulls back the registers and
			;returns to the interrupt.

			;alternatively, we could write the internal interrupt handler pointed to by $BFFE directly, but in this case, the registers aren't saved,
			;so we either have to save them in the program itself or throw them away in each interrupt, which probably is only feasible if there's
			;no processing outside the interrupt. In this case we could simply end the interrupt routine by doing RTI.
	
			;In our case, the complete music handling runs in the interrupt routine.

	.word Internal_Interrupt_Handler

	org $BFF0	;this is the table for initializing the VDP on startup which gets read by the BIOS on startup

	;the following values appear in the Creativision Pac Man clone
	;.byte $00	;VDP Reg. 0 -> External Video off, M2=0 -> Graphics Mode
	;.byte $E2	;VDP Reg. 1 -> 16K Mode; Screen active; Generate Interrupts on; M1=0; M3=0; Sprites = 16x16 unmagnified
	;.byte $04	;VDP Reg.2 -> Pattern Name Table = 04 * $400 = $1000
	;.byte $4E	;VDP Reg.3 -> Color Table Adress = 00 01001110 000000 = $1380
	;.byte $00	;VDP Reg.4 -> Pattern Generator Table = $0000
	;.byte $26	;VDP Reg.5 -> Sprite Attribute Table = 0 00100110 0000000 = $1300
	;.byte $01	;VDP Reg.6 -> Sprite Generator Table = 00001 00000000000 = $0800
	;.byte $01	;VDP Reg.7 -> TC=0; BD=1 -> Text Color = 0 (Transparent); Backdrop = 1 (Black)

	;And below are the values actually used by our program. Remember: the BIOS will ALWAYS read those values from $BFF0-$BFF7 and store them in the VDP registers
	;This happens BEFORE the BIOS first jumps into the cartridge.
	;The display is first set to NOT active (= blanked) and will be set to active later after the data has been written
	;Thus the original content of Reg. 1 has been commented out
	
	.byte $02	;VDP Reg. 0 -> External Video off, M2=1 -> Bitmap mode
;	.byte $E2	;VDP Reg. 1 -> 16K Mode; Screen active; Generate Interrupts on; M1=0; M3=0; Sprites = 16x16 unmagnified
	.byte $A2	;VDP Reg. 1 -> 16K Mode; Screen not active; Generate Interrupts on; M1=0; M3=0; Sprites = 16x16 unmagnified
	.byte $06	;VDP Reg.2 -> Pattern Name Table = 06 * $400 = $1800
	.byte $FF	;VDP Reg.3 -> Color Table Adress = $2000 (Bits 0-6 are an and mask, so they were set to 1 here)
	.byte $03	;VDP Reg.4 -> Pattern Generator Table = $0000; Bits 0-1 are an AND mask
	.byte $36	;VDP Reg.5 -> Sprite Attribute Table = 0 00110110 0000000 = $1B00
	.byte $07	;VDP Reg.6 -> Sprite Generator Table = 00111 00000000000 = $3800
	.byte $01	;VDP Reg.7 -> TC=0; BD=1 -> Text Color = 0 (Transparent); Backdrop = 1 (Black)

;This gives the following VDP memory map for our program:
;	$0000-$17FF	Pattern generator table
;	$1800-$1AFF	Pattern name table
;	$1B00-$1B7F	Sprite attribute table
;	$2000-$37FF	Color table
;	$3800-$3FFF	Sprite Generator Table
; ($1B80-$1FFF free)
;Now come the vectors the BIOS uses to jump into the cartridge. These work the same as the "real" 6502 vectors at $FFFC and $FFFE, which are located inside the BIOS.

	org $BFFC

;First we have the vector for the main program. This gets jumped to by the BIOS after basic initialisation. More on that has been said at the start of the program.
	
	.word Start

;Then there comes the vector for the interrupt handler. We already described this above. In this case we jump to the BIOS interrupt handler which saves all the registers
;and then jumps into the cartridge using the vector at $BFEA (see above).

	.word BIOS_Interrupt_Handler

;Now here comes a trick... the BIN file is read in by emulators in a slightly confusing way... the first 8K are $A000-$BFFF,
;then the next 8K are $8000-$9FFF. Because DASM doesn't support jumbling BIN's like this, we let it assemble the data to $C000.
;Since it's only data, it actually ends up being loaded to $8000, which is where we read it from and copy it into the VDP.

	org $C000
	include	ColorTable.asm	;Colortable is linked in at $C000, but actually is read to $8000
	org $DFFF
	.byte $00